@@ -8,6 +8,7 @@
 #                            and add new marker-related routines
 #               06/21/2006 - P. Harvey Patch to work with negative offsets
 #               07/07/2006 - P. Harvey Added support for 16-bit pointers
+#               02/19/2013 - P. Harvey Added IsEmpty()
 #
 # Data Members:
 #
@@ -17,6 +18,18 @@
 #   Pointers  - Hash of references to fixup pointer arrays, keyed by ByteOrder
 #               string (with "2" added if pointer is 16-bit [default is 32-bit],
 #               plus "_$marker" suffix if tagged with a marker name).
+#
+# Procedure:
+#
+#            1. Create a Fixup object for each data block containing pointers
+#            2. Call AddFixup with the offset of each pointer in the block
+#               - pointer is assumed int32u with the current byte order
+#               - may also be called with a fixup reference for contained blocks
+#            3. Add the necessary pointer offset to $$fixup{Shift}
+#            4. Add data size to $$fixup{Start} if data is added before the block
+#               - automatically also shifts pointers by this amount
+#            5. Call ApplyFixup to apply the fixup to all pointers
+#               - resets Shift and Start to 0 after applying fixup
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Fixup;
@@ -26,7 +39,7 @@ use Image::ExifTool qw(GetByteOrder SetByteOrder Get32u Get32s Set32u
                        Get16u Get16s Set16u);
 use vars qw($VERSION);
 
-$VERSION = '1.04';
+$VERSION = '1.05';
 
 sub AddFixup($$;$$);
 sub ApplyFixup($$);
@@ -167,6 +180,24 @@ sub ApplyFixup($$)
     $self->{Start} = $self->{Shift} = 0;
 }
 
+#------------------------------------------------------------------------------
+# Is this Fixup empty?
+# Inputs: 0) Fixup object ref
+# Returns: True if there are no offsets to fix
+sub IsEmpty($)
+{
+    my $self = shift;
+    my $phash = $self->{Pointers};
+    if ($phash) {
+        my $key;
+        foreach $key (keys %$phash) {
+            next unless ref $$phash{$key} eq 'ARRAY';
+            return 0 if @{$$phash{$key}};
+        }
+    }
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Does specified marker exist?
 # Inputs: 0) Fixup object reference, 1) marker name
@@ -303,6 +334,8 @@ Image::ExifTool::Fixup - Utility to handle pointer fixups
 
     $fixup->Dump();               # dump debugging information
 
+    $fixup->IsEmpty();            # return true if no offsets to fix
+
 =head1 DESCRIPTION
 
 This module contains the code to keep track of pointers in memory and to
@@ -321,7 +354,7 @@ linear list when ApplyFixups() is called.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
