@@ -11,8 +11,9 @@ package Image::ExifTool::SonyIDC;
 use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
+use Image::ExifTool::Exif;
 
-$VERSION = '1.03';
+$VERSION = '1.05';
 
 # Sony IDC tags (ref PH)
 %Image::ExifTool::SonyIDC::Main = (
@@ -229,6 +230,7 @@ $VERSION = '1.03';
 %Image::ExifTool::SonyIDC::Composite = (
     GROUPS => { 2 => 'Image' },
     IDCPreviewImage => {
+        Groups => { 2 => 'Preview' },
         Require => {
             0 => 'IDCPreviewStart',
             1 => 'IDCPreviewLength',
@@ -272,8 +274,8 @@ sub ExtractPreviews($)
             next;
         }
         # run through IDC preview images in the same order they were extracted
-        my $off = $et->GetValue($key) or last;
-        my $len = $et->GetValue("IDCPreviewLength$xtra") or last;
+        my $off = $et->GetValue($key, 'ValueConv') or last;
+        my $len = $et->GetValue("IDCPreviewLength$xtra", 'ValueConv') or last;
         # get stack version from number in group 1 name
         my $grp1 = $et->GetGroup($key, 1);
         if ($grp1 =~ /(\d+)$/) {
@@ -281,7 +283,7 @@ sub ExtractPreviews($)
             unless ($Image::ExifTool::Extra{$tag}) {
                 AddTagToTable(\%Image::ExifTool::Extra, $tag, {
                     Name => $tag,
-                    Groups => { 0 => 'Composite', 1 => 'Composite', 2 => 'Image'},
+                    Groups => { 0 => 'Composite', 1 => 'Composite', 2 => 'Preview'},
                 });
             }
             my $val = Image::ExifTool::Exif::ExtractImage($et, $off, $len, $tag);
